// Class: Histogram
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import java.awt.Color;
import java.util.Random;

/**
 *  Environment-Based Applications:<br>
 *
 *    The Histogram class constructs a histogram of a set of integer values.
 *
 *  @author Alyce Brady
 *  @version 1 December 2002
 **/
public class Histogram
{
    // Specify dimensions of environment display and individual cell size.
    private static final int MAX_WIDTH = 800;
    private static final int MAX_HEIGHT = 600;
    private static final int MIN_CELL_SIZE = 10;

    // Specify the environment's background color and highlight color.
    private static final Color BACKGROUND_COLOR = new Color(0, 155, 255);
    private static final Color HIGHLIGHT_COLOR = Color.red;

    // Keep track of basic information for this histogram.
    private BoundedEnv grid;
    private SettableEnvDisplay display;
    private int scaleFactor;
    private int cellSize = MIN_CELL_SIZE;

    /**
     *  Constructs a histogram for a certain number of values.
     *  (Precondition: <code>numValues < 20</code>.)
     *  @param title     title for top of window displaying histogram
     *  @param numValues the number of values to plot
     *  @param maxValue  the maximum value that could be plotted;
     *                   the histogram is scaled so that a full row
     *                   represents this value
     **/
    public Histogram(String title, int numValues, int maxValue)
    {
        // Determine the number of rows and columns to display.
        int numRows = numValues * 3 + 4;
        int numCols = maxValue + 2;
        if ( numCols * MIN_CELL_SIZE > MAX_WIDTH )
            numCols = MAX_WIDTH / MIN_CELL_SIZE;
        else
            cellSize = Math.min(MAX_WIDTH/numCols, MAX_HEIGHT/numRows);
                                                               
        scaleFactor = maxValue / numCols;
        if ( scaleFactor == 0 )
            scaleFactor++;

        // Construct a grid in which to draw the histogram.
        grid = new BoundedEnv(numRows, numCols);

        // Construct a window to display a histogram and specify how to
        // display color blocks and text cells.
        display = new EnvAppFrame(title, BACKGROUND_COLOR, 
                                  numCols * cellSize, numRows * cellSize,
                                  cellSize);
        display.setEnv(grid);
        DisplayMap.associate("ColorBlock", new ColorBlockDisplay());
        DisplayMap.associate("TextCell", new TextCellDisplay());
    }

    /**
     *  Constructs a histogram for the specified values and displays it.
     *  (Precondition: <code>values.length < 20</code> and
     *   every value in <code>values</code> is less than or equal
     *   to <code>maxValue</code>.)
     *  @param title    title for top of window displaying histogram
     *  @param values   the list of values to plot
     *  @param maxValue the maximum value that could be plotted;
     *                  the histogram is scaled so that a full row
     *                  represents this value
     **/
    public Histogram(String title, int[] values, int maxValue)
    {
        this(title, values.length, maxValue);

        // Fill in the histogram.
        for ( int i = 0; i < values.length; i++ )
            plot(i, values[i]);
    }

    /** Plots a row in the histogram; assumes that the histogram was created
     *  with the first constructor.
     *  (Precondition: <code>0 <= row < numValues</code> and 
     *   <code>0 <= value <= maxValue</code>, where <code>numValues</code>
     *   and <code>maxValue</code> are the values specified when the histogram
     *   was constructed.)
     *  @param row the row in which a value is to be plotted
     *  @param value the value to be plotted
     **/
    public void plot(int row, int value)
    {
        plot((new Integer(value)).toString(), row, value);
    }

    /** Plots a labeled row in the histogram; assumes that the histogram was
     *  created with the first constructor.
     *  (Precondition: <code>0 <= row < numValues</code> and 
     *   <code>0 <= value <= maxValue</code>, where <code>numValues</code>
     *   and <code>maxValue</code> are the values specified when the histogram
     *   was constructed.  Also, <code>label</code> is a short enough string
     *   to be displayed in a single cell in the histogram grid.)
     *  @param label  the label for the row
     *  @param row the row in which a value is to be plotted
     *  @param value the value to be plotted
     **/
    public void plot(String label, int row, int value)
    {
        // Label row with the value being represented.
        new TextCell(label, grid, new Location(row * 3 + 2, 0));

        // Plot the bar that represents the value.
        for ( int block = 0; block < value; block+= scaleFactor )
            new ColorBlock(HIGHLIGHT_COLOR, grid, 
                           new Location(row * 3 + 2, (block / scaleFactor) + 2));

        display();
    }

    /** Label the "x-axis" of the histogram with the values represented by
     *  each column.
     **/
    public void labelColumns()
    {
        labelColumns(1);
    }

    /** Label the "x-axis" of the histogram with the values represented by
     *  the columns; which columns are labeled depends on the
     *  <code>periodicity</code>.  If the <code>periodicity</code> is 1
     *  then every column is labeled; if the <code>periodicity</code> is 2
     *  then every other column is labeled; it it's 3 then every third
     *  column is labeled; <i>etc.</i>  The first and last columns, however,
     *  are always labeled, regardless of the <code>periodicity</code>.
     **/
    public void labelColumns(int periodicity)
    {
        // Place the column labels.
        for ( int col = 2, val = scaleFactor; col < grid.numCols();
              col++, val += scaleFactor )
        {
            // Label only if first, last, or specified by periodicity.
            if ( (col - 2) % periodicity == 0 || col == grid.numCols() - 1 )
            {
                new TextCell((new Integer(val)).toString(),
                             grid, new Location(grid.numRows() - 1,
                                                (val / scaleFactor) + 1));
            }
        }
        display();
    }

    /** Displays the histogram.
     **/
    public void display()
    {
        display.showEnv();
    }

}
